@@ -8,7 +8,10 @@
 #include <unistd.h>
 
 #include <cstring>
+#include <fstream>
+#include <iomanip>
 #include <iostream>
+#include <list>
 #include <memory>
 #include <stdexcept>
 #include <string>
@@ -90,6 +93,8 @@
 #define XADC_VAUX13   0x1d
 #define XADC_VAUX14   0x1e
 #define XADC_VAUX15   0x1f
+#define XADC_TEMP_MAX 0x20
+#define XADC_TEMP_MIN 0x24
 #define XADC_SUPBOFFS 0x30
 #define XADC_ADCBOFFS 0x31
 #define XADC_ADCBGAIN 0x32
@@ -122,6 +127,9 @@
 #define XADC_ALARM14  0x5e
 #define XADC_ALARM15  0x5f
 
+#define XADC_VCC_MINOFFSET 0x24
+#define XADC_VCC_MAXOFFSET 0x20
+
 /* Boundary-scan instruction set based on the FPGA model */
 static std::map<std::string, std::map<std::string, std::vector<uint8_t>>>
 	ircode_mapping {
@@ -131,6 +139,7 @@ static std::map<std::string, std::map<std::string, std::vector<uint8_t>>>
 			{
 				{ "USER1",       {0x02} },
 				{ "USER2",       {0x03} },
+				{ "CFG_OUT",     {0x04} },
 				{ "CFG_IN",      {0x05} },
 				{ "USERCODE",    {0x08} },
 				{ "IDCODE",      {0x09} },
@@ -347,6 +356,8 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 		_fpga_family = KINTEXUSP_FAMILY;
 	} else if (family == "artixusp") {
 		_fpga_family = ARTIXUSP_FAMILY;
+	} else if (family == "virtexus") {
+		_fpga_family = VIRTEXUS_FAMILY;
 	} else if (family == "virtexusp") {
 		_fpga_family = VIRTEXUSP_FAMILY;
 		_ircode_map = ircode_mapping.at("virtexusp");
@@ -403,9 +414,11 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 
 			unsigned int v = 0;
 			for (int i = 0; i < TEMP_MEAS; i++) {
-				v += Xilinx::xadc_single(0);
+				v += Xilinx::xadc_single(XADC_TEMP);
 			}
 			double temp = ((v/(double)TEMP_MEAS) * 503.975)/(1 << 16) - 273.15;
+			double max_temp = (Xilinx::xadc_single(XADC_TEMP_MAX) * 503.975)/(1 << 16) - 273.15;
+			double min_temp = (Xilinx::xadc_single(XADC_TEMP_MIN) * 503.975)/(1 << 16) - 273.15;
 
 			unsigned int channel_values[32];
 			for (int ch = 0; ch < MAX_CHANNEL; ch++) {
@@ -420,16 +433,32 @@ Xilinx::Xilinx(Jtag *jtag, const std::string &filename,
 				channel_values[ch] = v;
 			}
 
+			double vccint = (Xilinx::xadc_single(XADC_VCCINT)>>4)/4096.0 * 3.0; // ref: UG480
+			double vccintmax = (Xilinx::xadc_single(XADC_VCCINT + XADC_VCC_MAXOFFSET)>>4)/4096.0 * 3.0; // ref: UG480
+			double vccintmin = (Xilinx::xadc_single(XADC_VCCINT + XADC_VCC_MINOFFSET)>>4)/4096.0 * 3.0; // ref: UG480
+
+			double vccaux = (Xilinx::xadc_single(XADC_VCCAUX)>>4)/4096.0 * 3.0; // ref: UG480
+			double vccauxmax = (Xilinx::xadc_single(XADC_VCCAUX + XADC_VCC_MAXOFFSET)>>4)/4096.0 * 3.0; // ref: UG480
+			double vccauxmin = (Xilinx::xadc_single(XADC_VCCAUX + XADC_VCC_MINOFFSET)>>4)/4096.0 * 3.0; // ref: UG480
+
 			/* output as JSON dict */
 			std::cout << "{";
-			std::cout << "\"temp\": " << temp << ", ";
+			std::cout << "\"temp\": " << temp << ", " << std::endl;;
+			std::cout << "    \"maxtemp\": " << max_temp << ", " << std::endl;;
+			std::cout << "    \"mintemp\": " << min_temp << ", " << std::endl;;
 			std::cout << "\"raw\":  {";
 			for (int ch = 0; ch < MAX_CHANNEL; ch++) {
 				std::cout << "\"" << ch << "\": " << channel_values[ch]
 					 << ((ch == MAX_CHANNEL - 1)? "}" : ", ");
 			}
+			std::cout << "," << std::endl;
+			std::cout << "\"vccint\": " << vccint << ", " << std::endl;
+			std::cout << "   \"maxvccint\": " << vccintmax << ", " << std::endl;
+			std::cout << "   \"minvccint\": " << vccintmin << ", " << std::endl;
+			std::cout << "\"vccaux\": " << vccaux << ", " << std::endl;
+			std::cout << "   \"maxvccaux\": " << vccauxmax << ", " << std::endl;
+			std::cout << "   \"minvccaux\": " << vccauxmin << ", " << std::endl;
 			std::cout << "}" << std::endl;
-
 		} else {
 			throw std::runtime_error("Error: read_xadc only supported for Artix 7");
 		}
@@ -807,8 +836,250 @@ void Xilinx::program_mem(ConfigBitstreamParser *bitfile)
 	 * now functional.                                    X     1   3
 	 */
 	_jtag->go_test_logic_reset();
-        /* Some xc7s50 does not detect correct connected flash w/o this shift*/
-        _jtag->shiftIR(tx_buf, rx_buf, _irlen);
+	/* Some xc7s50 does not detect correct connected flash w/o this shift*/
+	_jtag->shiftIR(tx_buf, rx_buf, _irlen);
+	uint8_t ir_c = rx_buf[0] & 0x03;
+	uint8_t isc_done = ((rx_buf[0] >> 2) & 0x01);
+	uint8_t isc_ena  = ((rx_buf[0] >> 3) & 0x01);
+	uint8_t init     = ((rx_buf[0] >> 4) & 0x01);
+	uint8_t done     = ((rx_buf[0] >> 5) & 0x01);
+	printf("Shift IR %02x\n", rx_buf[0]);
+	printf("ir: %x isc_done %x isc_ena %x init %x done %x\n", ir_c, isc_done, isc_ena,
+		init, done);
+
+	if (!done) {
+		read_register("STAT");
+	}
+}
+
+static uint32_t char_array_to_word(uint8_t *in)
+{
+	return (((uint32_t)in[3] << 24) |
+		((uint32_t)in[2] << 16) |
+		((uint32_t)in[1] <<  8) |
+		((uint32_t)in[0] <<  0));
+}
+
+typedef struct {
+	std::string description;
+	uint8_t offset;
+	uint8_t size;
+	std::map<int, std::string> reg_cnt;
+} reg_struct_t;
+
+#define REG_ENTRY(_description, _offset, _size, ...) \
+	{_description, _offset, _size, {__VA_ARGS__}}
+
+static const std::map<std::string, std::list<reg_struct_t>> reg_content = {
+	{"CTRL0", std::list<reg_struct_t>{
+		REG_ENTRY("GTS USR B",       0, 1,
+			{0, "I/Os 3-stated"}, {1, "I/Os active"}),
+		REG_ENTRY("Reserved",        1, 2),
+		REG_ENTRY("PERSIST",         3, 1,
+			{0, "No"}, {1, "Yes"}),
+		REG_ENTRY("SBITS",           4, 2,
+			{0, "Read/Write OK"}, {1, "Readback disabled"},
+			{2, "Both Writes and Reads disabled"},
+			{3, "Both Writes and Reads disabled"}),
+		REG_ENTRY("DEC",             6, 1,
+			{0, "Disable"}, {1, "Enable"}),
+		REG_ENTRY("FARSRC",          7, 1),
+		REG_ENTRY("GLUMASK B",       8, 1),
+		REG_ENTRY("Reserved",        9, 1),
+		REG_ENTRY("ConfigFallback", 10, 1,
+			{0, "Disable"}, {1, "Enable"}),
+		REG_ENTRY("Reserved",       11, 1),
+		REG_ENTRY("OverTempPwrDown",12, 1,
+			{0, "Disable"}, {1, "Enable"}),
+		REG_ENTRY("Reserved",       13, 17),
+		REG_ENTRY("ICAP Select",    30, 1,
+			{0, "Top ICAPE2 Port Enabled"},
+			{1, "Bottom ICAPE2 Port Enabled"}),
+		REG_ENTRY("EFUSE key",      31, 1,
+			{0, "Battery backed RAM"}, {1, "eFUSE"}),
+	}},
+	{"STAT", std::list<reg_struct_t>{
+		REG_ENTRY("CRC Error",       0, 1,
+			{0, "No CRC error"}, {1, "CRC error"}),
+		REG_ENTRY("Part Secured",    1, 1),
+		REG_ENTRY("MMCM lock",       2, 1),
+		REG_ENTRY("DCI match",       3, 1),
+		REG_ENTRY("EOS",             4, 1),
+		REG_ENTRY("GTS CFG B",       5, 1),
+		REG_ENTRY("GWE",             6, 1),
+		REG_ENTRY("GHIGH B",         7, 1),
+		REG_ENTRY("MODE",            8, 3),
+		REG_ENTRY("INIT Complete",  11, 1),
+		REG_ENTRY("INIT B",         12, 1),
+		REG_ENTRY("Release Done",   13, 1),
+		REG_ENTRY("Done",           14, 1),
+		REG_ENTRY("ID Error",       15, 1,
+			{0, "No ID error"}, {1, "ID error"}),
+		REG_ENTRY("DEC Error",      16, 1),
+		REG_ENTRY("XADC Over temp", 17, 1),
+		REG_ENTRY("STARTUP State",  18, 3),
+		REG_ENTRY("Reserved",       21, 4),
+		REG_ENTRY("BUS Width",      25, 2,
+			{0, "x1"}, {1, "x8"}, {2, "x16"}, {3, "x32"}),
+		REG_ENTRY("Reserved",       27, 5)
+	}},
+	// UG470 Table 5-34
+	{"WBSTAR", std::list<reg_struct_t>{
+		// Next bitstream start address. The default start address
+		//   is address zero.
+		REG_ENTRY("START ADDR",  0, 29),
+		REG_ENTRY("RS TS B",    29, 1,
+			{0, "3-state enabled (RS[1:0] disabled) (default)"},
+			{1, "3-state disabled (RS[1:0] enabled)"},
+		),
+		// RS[1:0] pin value on next warm boot. The default is 00.
+		REG_ENTRY("RS[1:0]",    30, 2),
+	}},
+	// UG470 Table 5-39
+	{"BOOTSTS", std::list<reg_struct_t>{
+		// Status 0 is valid
+		REG_ENTRY("VALID 0",       0, 1),
+		REG_ENTRY("FALLBACK 0",    1, 1,
+			{0, "Normal configuration"},
+			{1, "Fallback to default reconfiguration, RS[1:0] actively drives 2'b00"}
+		),
+		// Internal PROG triggered configuration
+		REG_ENTRY("IPROG 0",       2, 1),
+		// Watchdog time-out error
+		REG_ENTRY("WTO Error 0",   3, 1),
+		// ID_CODE error
+		REG_ENTRY("ID Error 0",    4, 1),
+		// CRC error
+		REG_ENTRY("CRC Error 0",   5, 1),
+		// BPI address counter wraparound error, supported in
+		//   asynchronous read mode
+		REG_ENTRY("WRAP Error 0",  6, 1),
+		// HMAC error
+		REG_ENTRY("HMAC Error 0",  7, 1),
+		REG_ENTRY("VALID 1",       8, 1),
+		REG_ENTRY("FALLBACK 1",    9, 1,
+			{0, "Normal configuration"},
+			{1, "Fallback to default reconfiguration, RS[1:0] actively drives 2'b00"}
+		),
+		REG_ENTRY("IPROG 1",      10, 1),
+		REG_ENTRY("WTO Error 1",  11, 1),
+		REG_ENTRY("ID Error 1",   12, 1),
+		REG_ENTRY("CRC Error 1",  13, 1),
+		REG_ENTRY("WRAP Error 1", 14, 1),
+		REG_ENTRY("HMAC Error 1", 15, 1),
+	}},
+};
+
+/* UG470 table 5-23 */
+static const std::map<std::string, uint8_t> reg_code = {
+	{"CTRL0",   0x05},  // Control register 0
+	{"STAT",    0x07},  // Status register
+	{"CONF0",   0x09},  // Configuration Option 0
+	{"CONF1",   0x0e},  // Configuration Option 1
+	{"WBSTAR",  0x10},  // Warm Boot Start Address Register
+	{"BOOTSTS", 0x16},  // Boot history status register
+	{"CTRL1",   0x18},  // Control register 1
+};
+
+uint32_t Xilinx::dumpRegister(std::string reg_name)
+{
+	uint8_t reg[4];
+	const uint8_t dummy[] = {0x00, 0x00, 0x00, 0x04};
+	/* opcode:
+	 * 0x0: NOP
+	 * 0x1: read
+	 * 0x2: write
+	 * 0x3: Reserved
+	 */
+	/* register code
+	 * 0x05: Control register 0
+	 * 0x07: Status register
+	 * 0x09: Configuration Option 0
+	 * 0x0e: Configuration Option 1
+	 * 0x18: Control Register 1
+	 * 0x16: Boot history Status register
+	 */
+	auto code = reg_code.find(reg_name);
+	if (code == reg_code.end()) {
+		printError("Unknown register " + reg_name);
+		printError("Known Register are:");
+		for (auto reg :reg_code)
+			printError("\t" + reg.first);
+		return 0xdeadbeef;
+	}
+
+	/* packet type 1 */
+	const uint32_t regcode = static_cast<uint32_t>(code->second);
+	uint32_t cfg_packets[] = {
+		0xAA995566,  // Sync Word
+		0x20000000,  // NOOP
+		((0x01    & 0x0007) << 29) |  // [31:29]: Header type
+		((0x01    & 0x0003) << 27) |  // [28:27]: opcode
+		((regcode & 0x3fff) << 13) |  // [26:13]: register code
+		((0x00    & 0x0003) << 11) |  // [12:11]: Reserved must be set to 0
+		((0x01    & 0x07ff) <<  0),   // [10:0 ]: word count
+		0x20000000,  // NOOP
+		0x20000000,  // NOOP
+	};
+
+	_jtag->go_test_logic_reset();
+	_jtag->set_state(Jtag::RUN_TEST_IDLE);
+	_jtag->shiftIR(get_ircode(_ircode_map, "CFG_IN"), NULL, _irlen, Jtag::SELECT_DR_SCAN);
+
+	Jtag::tapState_t next_state = Jtag::SHIFT_DR;
+	for (int i = 0; i < 5; i++) {
+		if (i == 4)
+			next_state = Jtag::SELECT_IR_SCAN;
+		const uint32_t tmp = BitParser::reverse_32(cfg_packets[i]);
+		const uint8_t cfg[] = {
+			static_cast<uint8_t>((tmp >>  0) & 0xff),
+			static_cast<uint8_t>((tmp >>  8) & 0xff),
+			static_cast<uint8_t>((tmp >> 16) & 0xff),
+			static_cast<uint8_t>((tmp >> 24) & 0xff)
+		};
+
+		_jtag->shiftDR(cfg, NULL, 32, next_state);
+	}
+
+	_jtag->shiftIR(get_ircode(_ircode_map, "CFG_OUT"), NULL, _irlen, Jtag::SELECT_DR_SCAN);
+	_jtag->shiftDR(dummy, reg, 32);
+	_jtag->go_test_logic_reset();
+
+	uint32_t reg_word = char_array_to_word(reg);
+	reg_word = BitParser::reverse_32(reg_word);
+	return reg_word;
+}
+
+void Xilinx::displayRegister(const std::string reg_name, const uint32_t reg_val) {
+	auto reg = reg_content.find(reg_name);
+	if (reg == reg_content.end()) {
+		printError("Unknown register " + reg_name);
+		return;
+	}
+
+	std::stringstream raw_val;
+	raw_val << "0x" << std::hex << reg_val;
+	printSuccess("Register raw value: " + raw_val.str());
+
+	const std::list<reg_struct_t> regs = reg->second;
+	for (reg_struct_t r: regs) {
+		uint8_t offset = r.offset;
+		uint8_t size = r.size;
+		uint32_t mask = (1 << size) - 1;
+		uint32_t val = (reg_val >> offset) & mask;
+		std::stringstream ss, desc;
+		desc << r.description;
+		ss << std::setw(16) << std::left << r.description;
+		if (r.reg_cnt.size() != 0) {
+			ss << r.reg_cnt[val];
+		} else {
+			std::stringstream hex_val;
+			hex_val << "0x" << std::hex << val;
+			ss << hex_val.str();
+		}
+
+		printInfo(ss.str());
+	}
 }
 
 bool Xilinx::dumpFlash(uint32_t base_addr, uint32_t len)
@@ -862,6 +1133,21 @@ bool Xilinx::dumpFlash(uint32_t base_addr, uint32_t len)
 	return true;
 }
 
+bool Xilinx::detect_flash()
+{
+	if (_flash_chips & PRIMARY_FLASH) {
+		select_flash_chip(PRIMARY_FLASH);
+		if (!SPIInterface::detect_flash())
+			return false;
+	}
+	if (_flash_chips & SECONDARY_FLASH) {
+		select_flash_chip(SECONDARY_FLASH);
+		if (!SPIInterface::detect_flash())
+			return false;
+	}
+	return true;
+}
+
 bool Xilinx::protect_flash(uint32_t len)
 {
 	if (_flash_chips & PRIMARY_FLASH) {
@@ -892,6 +1178,21 @@ bool Xilinx::unprotect_flash()
 	return true;
 }
 
+bool Xilinx::set_quad_bit(bool set_quad)
+{
+	if (_flash_chips & PRIMARY_FLASH) {
+		select_flash_chip(PRIMARY_FLASH);
+		if (!SPIInterface::set_quad_bit(set_quad))
+			return false;
+	}
+	if (_flash_chips & SECONDARY_FLASH) {
+		select_flash_chip(SECONDARY_FLASH);
+		if (!SPIInterface::set_quad_bit(set_quad))
+			return false;
+	}
+	return true;
+}
+
 bool Xilinx::bulk_erase_flash()
 {
 	if (_flash_chips & PRIMARY_FLASH) {
@@ -1488,7 +1789,10 @@ void Xilinx::xc2c_init(uint32_t idcode)
 {
 	_fpga_family = XC2C_FAMILY;
 	std::string model = fpga_list[idcode].model;
-	int underscore_pos = model.find_first_of('_', 0);
+	size_t underscore_pos = model.find_first_of('_', 0);
+	if (underscore_pos == model.npos) {
+		underscore_pos = model.length();
+	}
 	snprintf(_cpld_base_name, underscore_pos,
 			"%s", model.substr(0, underscore_pos).c_str());
 	switch ((idcode >> 16) & 0x3f) {
